@@ -103,6 +103,12 @@ prefix_copy_index_vector_map = {
     'c': _tools.ccopy_index_vector,
     'z': _tools.zcopy_index_vector
 }
+prefix_compute_smoothed_state_weights_map = {
+    's': _tools._scompute_smoothed_state_weights,
+    'd': _tools._dcompute_smoothed_state_weights,
+    'c': _tools._ccompute_smoothed_state_weights,
+    'z': _tools._zcompute_smoothed_state_weights
+}
 
 
 def set_mode(compatibility=None):
@@ -194,7 +200,7 @@ def companion_matrix(polynomial):
             raise ValueError("Companion matrix polynomials must include at"
                              " least two terms.")
 
-        if isinstance(polynomial, list) or isinstance(polynomial, tuple):
+        if isinstance(polynomial, (list, tuple)):
             try:
                 # Note: cannot use polynomial[0] because of the special
                 # behavior associated with matrix polynomials and the constant
@@ -250,9 +256,9 @@ def diff(series, k_diff=1, k_seasonal_diff=None, seasonal_periods=1):
     ----------
     series : array_like
         The series to be differenced.
-    diff : int, optional
+    k_diff : int, optional
         The number of simple differences to perform. Default is 1.
-    seasonal_diff : int or None, optional
+    k_seasonal_diff : int or None, optional
         The number of seasonal differences to perform. Default is no seasonal
         differencing.
     seasonal_periods : int, optional
@@ -1857,7 +1863,12 @@ def prepare_trend_spec(trend):
         if trend.ndim > 0:
             polynomial_trend = (trend > 0).astype(int)
         else:
-            raise ValueError('Invalid trend method.')
+            raise ValueError(
+                "Valid trend inputs are 'c' (constant), 't' (linear trend in "
+                "time), 'ct' (both), 'ctt' (both with trend squared) or an "
+                "interable defining a polynomial, e.g., [1, 1, 0, 1] is `a + "
+                f"b*t + ct**3`. Received {trend}"
+            )
 
     # Note: k_trend is not the degree of the trend polynomial, because e.g.
     # k_trend = 1 corresponds to the degree zero polynomial (with only a
@@ -1892,3 +1903,333 @@ def _safe_cond(a):
             return np.nan
         else:
             return np.inf
+
+
+def _compute_smoothed_state_weights(ssm, compute_t=None, compute_j=None,
+                                    compute_prior_weights=None, scale=1.0):
+    # Get references to the Cython objects
+    _model = ssm._statespace
+    _kfilter = ssm._kalman_filter
+    _smoother = ssm._kalman_smoother
+
+    # Determine the appropriate function for the dtype
+    func = prefix_compute_smoothed_state_weights_map[ssm.prefix]
+
+    # Handle compute_t and compute_j indexes
+    if compute_t is None:
+        compute_t = np.arange(ssm.nobs)
+    if compute_j is None:
+        compute_j = np.arange(ssm.nobs)
+    compute_t = np.unique(np.atleast_1d(compute_t).astype(np.int32))
+    compute_t.sort()
+    compute_j = np.unique(np.atleast_1d(compute_j).astype(np.int32))
+    compute_j.sort()
+
+    # Default setting for computing the prior weights
+    if compute_prior_weights is None:
+        compute_prior_weights = compute_j[0] == 0
+    # Validate that compute_prior_weights is valid
+    if compute_prior_weights and compute_j[0] != 0:
+        raise ValueError('If `compute_prior_weights` is set to True, then'
+                         ' `compute_j` must include the time period 0.')
+
+    # Compute the weights
+    weights, state_intercept_weights, prior_weights, _ = func(
+        _smoother, _kfilter, _model, compute_t, compute_j, scale,
+        bool(compute_prior_weights))
+
+    # Re-order missing entries correctly and transpose to the appropriate
+    # shape
+    if np.any(_model.nmissing):
+        shape = weights.shape
+        # Transpose m, p, t, j, -> t, m, p, j so that we can use the
+        # `reorder_missing_matrix` function
+        weights = np.asfortranarray(weights.transpose(2, 0, 1, 3).reshape(
+            shape[2] * shape[0], shape[1], shape[3], order='C'))
+        missing = np.asfortranarray(np.isnan(ssm.endog).astype(np.int32))
+        reorder_missing_matrix(weights, missing, reorder_cols=True,
+                               inplace=True)
+        # Transpose t, m, p, j -> t, j, m, p,
+        weights = (weights.reshape(shape[2], shape[0], shape[1], shape[3])
+                          .transpose(0, 3, 1, 2))
+    else:
+        # Transpose m, p, t, j -> t, j, m, p
+        weights = weights.transpose(2, 3, 0, 1)
+
+    # Transpose m, l, t, j -> t, j, m, l
+    state_intercept_weights = state_intercept_weights.transpose(2, 3, 0, 1)
+
+    # Transpose m, l, t -> t, m, l
+    prior_weights = prior_weights.transpose(2, 0, 1)
+
+    return weights, state_intercept_weights, prior_weights
+
+
+def compute_smoothed_state_weights(results, compute_t=None, compute_j=None,
+                                   compute_prior_weights=None, resmooth=None):
+    r"""
+    Construct the weights of observations and the prior on the smoothed state
+
+    Parameters
+    ----------
+    results : MLEResults object
+        Results object from fitting a state space model.
+    compute_t : array_like, optional
+        An explicit list of periods `t` of the smoothed state vector to compute
+        weights for (see the Returns section for more details about the
+        dimension `t`). Default is to compute weights for all periods `t`.
+        However, if weights for only a few time points are desired, then
+        performance can be improved by specifying this argument.
+    compute_j : array_like, optional
+        An explicit list of periods `j` of observations to compute
+        weights for (see the Returns section for more details about the
+        dimension `j`). Default is to compute weights for all periods `j`.
+        However, if weights for only a few time points are desired, then
+        performance can be improved by specifying this argument.
+    compute_prior_weights : bool, optional
+        Whether or not to compute the weight matrices associated with the prior
+        mean (also called the "initial state"). Note that doing so requires
+        that period 0 is in the periods defined in `compute_j`. Default is True
+        if 0 is in `compute_j` (or if the `compute_j` argument is not passed)
+        and False otherwise.
+    resmooth : bool, optional
+        Whether or not to re-perform filtering and smoothing prior to
+        constructing the weights. Default is to resmooth if the smoothed_state
+        vector is different between the given results object and the
+        underlying smoother. Caution is adviced when changing this setting.
+        See the Notes section below for more details.
+
+    Returns
+    -------
+    weights : array_like
+        Weight matrices that can be used to construct the smoothed state from
+        the observations. The returned matrix is always shaped
+        `(nobs, nobs, k_states, k_endog)`, and entries that are not computed
+        are set to NaNs. (Entries will not be computed if they are not
+        included in `compute_t` and `compute_j`, or if they correspond to
+        missing observations, or if they are for periods in which the exact
+        diffuse Kalman filter is operative). The `(t, j, m, p)`-th element of
+        this matrix contains the weight of the `p`-th element of the
+        observation vector at time `j` in constructing the `m`-th element of
+        the smoothed state vector at time `t`.
+    prior_weights : array_like
+        Weight matrices that describe the impact of the prior (also called the
+        initialization) on the smoothed state vector. The returned matrix is
+        always shaped `(nobs, k_states, k_states)`. If prior weights are not
+        computed, then all entries will be set to NaNs. The `(t, m, l)`-th
+        element of this matrix contains the weight of the `l`-th element of the
+        prior mean (also called the "initial state") in constructing the
+        `m`-th element of the smoothed state vector at time `t`.
+
+    Notes
+    -----
+    In [1]_, Chapter 4.8, it is shown how the smoothed state vector can be
+    written as a weighted vector sum of observations:
+
+    .. math::
+
+        \hat \alpha_t = \sum_{j=1}^n \omega_{jt}^{\hat \alpha} y_j
+
+    One output of this function is the weights
+    :math:`\omega_{jt}^{\hat \alpha}`. Note that the description in [1]_
+    assumes that the prior mean (or "initial state") is fixed to be zero. More
+    generally, the smoothed state vector will also depend partly on the prior.
+    The second output of this function are the weights of the prior mean.
+
+    There are two important technical notes about the computations used here:
+
+    1. In the univariate approach to multivariate filtering (see e.g.
+       Chapter 6.4 of [1]_), all observations are introduced one at a time,
+       including those from the same time period. As a result, the weight of
+       each observation can be different than when all observations from the
+       same time point are introduced together, as in the typical multivariate
+       filtering approach. Here, we always compute weights as in the
+       multivariate filtering approach, and we handle singular forecast error
+       covariance matrices by using a pseudo-inverse.
+    2. Constructing observation weights for periods in which the exact diffuse
+       filter (see e.g. Chapter 5 of [1]_) is operative is not done here, and
+       so the corresponding entries in the returned weight matrices will always
+       be set equal to zeros. While handling these periods may be implemented
+       in the future, one option for constructing these weights is to use an
+       approximate (instead of exact) diffuse initialization for this purpose.
+
+    Finally, one note about implementation: to compute the weights, we use
+    attributes of the underlying filtering and smoothing Cython objects
+    directly. However, these objects are not frozen with the result
+    computation, and we cannot guarantee that their attributes have not
+    changed since `res` was created. As a result, by default we re-run the
+    filter and smoother to ensure that the attributes there actually correspond
+    to the `res` object. This can be overridden by the user for a small
+    performance boost if they are sure that the attributes have not changed;
+    see the `resmooth` argument.
+
+    References
+    ----------
+    .. [1] Durbin, James, and Siem Jan Koopman. 2012.
+            Time Series Analysis by State Space Methods: Second Edition.
+            Oxford University Press.
+    """
+    # Get the python model object
+    mod = results.model
+    # Always update the parameters to be consistent with `res`
+    mod.update(results.params)
+    # By default, resmooth if it appears the results have changed; check is
+    # based on the smoothed state vector
+    if resmooth is None:
+        resmooth = np.any(results.smoothed_state !=
+                          mod.ssm._kalman_smoother.smoothed_state)
+    # Resmooth if necessary, otherwise at least update the Cython model
+    if resmooth:
+        mod.ssm.smooth(conserve_memory=0, update_representation=False,
+                       update_filter=False, update_smoother=False)
+    else:
+        mod.ssm._initialize_representation()
+
+    return _compute_smoothed_state_weights(
+        mod.ssm, compute_t=compute_t, compute_j=compute_j,
+        compute_prior_weights=compute_prior_weights,
+        scale=results.filter_results.scale)
+
+
+def get_impact_dates(previous_model, updated_model, impact_date=None,
+                     start=None, end=None, periods=None):
+    """
+    Compute start/end periods and an index, often for impacts of data updates
+
+    Parameters
+    ----------
+    previous_model : MLEModel
+        Model used to compute default start/end periods if None are given.
+        In the case of computing impacts of data updates, this would be the
+        model estimated with the previous dataset. Otherwise, can be the same
+        as `updated_model`.
+    updated_model : MLEModel
+        Model used to compute the index. In the case of computing impacts of
+        data updates, this would be the model estimated with the updated
+        dataset. Otherwise, can be the same as `previous_model`.
+    impact_date : {int, str, datetime}, optional
+        Specific individual impact date. Cannot be used in combination with
+        `start`, `end`, or `periods`.
+    start : {int, str, datetime}, optional
+        Starting point of the impact dates. If given, one of `end` or `periods`
+        must also be given. If a negative integer, will be computed relative to
+        the dates in the `updated_model` index. Cannot be used in combination
+        with `impact_date`.
+    end : {int, str, datetime}, optional
+        Ending point of the impact dates. If given, one of `start` or `periods`
+        must also be given. If a negative integer, will be computed relative to
+        the dates in the `updated_model` index. Cannot be used in combination
+        with `impact_date`.
+    periods : int, optional
+        Number of impact date periods. If given, one of `start` or `end`
+        must also be given. Cannot be used in combination with `impact_date`.
+
+    Returns
+    -------
+    start : int
+        Integer location of the first included impact dates.
+    end : int
+        Integer location of the last included impact dates (i.e. this integer
+        location is included in the returned `index`).
+    index : pd.Index
+        Index associated with `start` and `end`, as computed from the
+        `updated_model`'s index.
+
+    Notes
+    -----
+    This function is typically used as a helper for standardizing start and
+    end periods for a date range where the most sensible default values are
+    based on some initial dataset (here contained in the `previous_model`),
+    while index-related operations (especially relative start/end dates given
+    via negative integers) are most sensibly computed from an updated dataset
+    (here contained in the `updated_model`).
+
+    """
+    # There doesn't seem to be any universal default that both (a) make
+    # sense for all data update combinations, and (b) work with both
+    # time-invariant and time-varying models. So we require that the user
+    # specify exactly two of start, end, periods.
+    if impact_date is not None:
+        if not (start is None and end is None and periods is None):
+            raise ValueError('Cannot use the `impact_date` argument in'
+                             ' combination with `start`, `end`, or'
+                             ' `periods`.')
+        start = impact_date
+        periods = 1
+    if start is None and end is None and periods is None:
+        start = previous_model.nobs - 1
+        end = previous_model.nobs - 1
+    if int(start is None) + int(end is None) + int(periods is None) != 1:
+        raise ValueError('Of the three parameters: start, end, and'
+                         ' periods, exactly two must be specified')
+    # If we have the `periods` object, we need to convert `start`/`end` to
+    # integers so that we can compute the other one. That's because
+    # _get_prediction_index doesn't support a `periods` argument
+    elif start is not None and periods is not None:
+        start, _, _, _ = updated_model._get_prediction_index(start, start)
+        end = start + (periods - 1)
+    elif end is not None and periods is not None:
+        _, end, _, _ = updated_model._get_prediction_index(end, end)
+        start = end - (periods - 1)
+    elif start is not None and end is not None:
+        pass
+
+    # Get the integer-based start, end and the prediction index
+    start, end, out_of_sample, prediction_index = (
+        updated_model._get_prediction_index(start, end))
+    end = end + out_of_sample
+
+    return start, end, prediction_index
+
+
+def _atleast_1d(*arys):
+    """
+    Version of `np.atleast_1d`, copied from
+    https://github.com/numpy/numpy/blob/master/numpy/core/shape_base.py,
+    with the following modifications:
+
+    1. It allows for `None` arguments, and passes them directly through
+    """
+    res = []
+    for ary in arys:
+        if ary is None:
+            result = None
+        else:
+            ary = np.asanyarray(ary)
+            if ary.ndim == 0:
+                result = ary.reshape(1)
+            else:
+                result = ary
+        res.append(result)
+    if len(res) == 1:
+        return res[0]
+    else:
+        return res
+
+
+def _atleast_2d(*arys):
+    """
+    Version of `np.atleast_2d`, copied from
+    https://github.com/numpy/numpy/blob/master/numpy/core/shape_base.py,
+    with the following modifications:
+
+    1. It allows for `None` arguments, and passes them directly through
+    2. Instead of creating new axis at the beginning, it creates it at the end
+    """
+    res = []
+    for ary in arys:
+        if ary is None:
+            result = None
+        else:
+            ary = np.asanyarray(ary)
+            if ary.ndim == 0:
+                result = ary.reshape(1, 1)
+            elif ary.ndim == 1:
+                result = ary[:, np.newaxis]
+            else:
+                result = ary
+        res.append(result)
+    if len(res) == 1:
+        return res[0]
+    else:
+        return res
